unit PPDraw;

// 26.08.01 - added flexible label formats, removed broken PatBlt (tris didn't rotate in Visio)
// 06.11.18 - removed broken hacks with ',' decimal separators

interface

uses Windows, SysUtils, MiscRoutines, Graphics, MainUnit, v_lib;

procedure DrawPPOnCanvas(pp:patchprofile_p;Canvas:TCanvas;Width,Height:integer;split:integer);

implementation

uses ChooseMdlUnit, FormatDlg;

procedure vec2PerpendicularLine(var d1,d2,da,p1,p2:vec2_t;len,offset:vec_t;split:integer);
var
  db : vec2_t;
  ofs : vec2_t;
begin
  vec2Sub(da,p2,p1);
  // -a2, a1
  d1[0]:=-da[1];
  d1[1]:=da[0];
  len:=len/2;
  vec2Unit(d1,d1);
  vec2Scale(ofs,d1,offset);
  vec2Scale(da,da,1/split);
  vec2Scale(db,d1,len);
  vec2Add(p1,p1,ofs);
  vec2Add(p2,p2,ofs);
  vec2Add(d2,p1,db);
  vec2Sub(d1,p1,db);
end;

procedure DrawPPOnCanvas(pp:patchprofile_p;Canvas:TCanvas;Width,Height:integer;split:integer);
var
  i,j,k,l : integer;
  clist : i_32_array_p;
  xb,yb:vec3_t;
  x,y,dx,dy:vec2_t;
  abc : vec3_t_array_p;
  lt : array[0..127] of char;
  plt : pchar;
const
  scales_dist : vec_t = 10; // TODO
  procedure DrawGridDir(Dest:TCanvas;x1,y1,x2,y2,x3,y3:vec_t;split:integer);
  var
    i : integer;
  begin
    // 1 -> 2-3
    dx[0]:=(x2-x1)/(split+1);
    dy[0]:=(y2-y1)/(split+1);
    dx[1]:=(x3-x1)/(split+1);
    dy[1]:=(y3-y1)/(split+1);
    x[0]:=x1+dx[0];y[0]:=y1+dy[0];
    x[1]:=x1+dx[1];y[1]:=y1+dy[1];
    for i:=1 to split do begin
      Dest.MoveTo(round(x[0]),round(y[0]));
      Dest.LineTo(round(x[1]),round(y[1]));
      x[0]:=x[0]+dx[0];
      y[0]:=y[0]+dy[0];
      x[1]:=x[1]+dx[1];
      y[1]:=y[1]+dy[1];
    end;
  end; // DrawGridDir;
  // Text centered at (x,y)
  procedure DrawLabel(Dest:TCanvas;x,y:vec_t;val:pChar);
  var
    w,h:vec_t;
    PenColor : TColor;
    Color : TColor;
  begin
    w:=Dest.TextWidth(val)/2.0;
    h:=Dest.TextHeight(val)/2.0;
    PenColor := Dest.Pen.Color;
    Color := Dest.Brush.Color;
    Dest.Pen.Color := CLWhite;
    Dest.Brush.Color := CLWhite;
    Dest.Rectangle(round(x-w),round(y-h),round(x+w),round(y+h));
    Dest.Pen.Color := PenColor;
    Dest.Brush.Color := Color;
    Dest.TextOut(round(x-w),round(y-h),val);
  end; // DrawLabel
var
  p1,p2,p,d1,d2,da : vec2_t;
  comma : char;
  s : string;
  len : vec_t;
  ofs : vec_t;
  Bound  : array[0..3] of TPoint;
  v1a,v1b,v1c,v2a,v2b,v2c,dva,dvb,dvc:vec_t;
  shiftx, shifty, zoomx, zoomy : vec_t;
const
  GlobalZoom : vec_t = 0.8; // global scale TODO
  aoscale : vec_t = 1.4; // offset factor for axes TODO
  LabelOffset : vec_t = 20; // offset for labels TODO
  tickslen : vec_t = 1.3; // offset for ticks TODO
  disdiv : vec_t = 20;
  lendiv : vec_t = 2;
begin
  try
    GlobalZoom := StrToFloat(FormatDialog.EGlobalZoom.Text);
    aoscale := StrToFloat(FormatDialog.Eaoscale.Text);
    LabelOffset := StrToFloat(FormatDialog.ELabelOffset.Text);
    tickslen := StrToFloat(FormatDialog.Etickslen.Text);
  except
    on EConvertError do
    begin
      Windows.MessageBox(MainForm.Handle,'   " "  "".',', , ',IDOK);
      exit;
    end;
  end;
  zoomx := GlobalZoom;
  zoomy := GlobalZoom;
  shiftx := Width * 0.5 * (1 - zoomx);
  shifty := Height * 0.5 * (1 - zoomy);
  abc:=@ChooseMdlParamsDialog.abc;
  // Determine decimal separator char for current locale
  comma := (PChar(FloatToStr(1.5))+1)^;
  with Canvas do
  begin
    for i:=0 to 2 do begin
      xb[i]:=pp^.base[i][0] * zoomx + shiftx;
      yb[i]:=pp^.base[i][1] * zoomy + shifty;
    end;
    if FormatDialog.CBFillB.Checked then begin
      Brush.Color:=FormatDialog.BackBox.Brush.Color;
      Windows.PatBlt(Handle,0,0,Width,Height,PATCOPY);
    end; { 26.08.01
    else
      Windows.PatBlt(Handle,0,0,Width,Height,WHITENESS); }
    // Triangle
    if FormatDialog.CBBound.Checked then begin
      Pen.Width:=FormatDialog.SEBoundW.Value;
      Pen.Color:=FormatDialog.BoundBox.Brush.Color;
      for i:=0 to 3 do begin
        Bound[i].x:=round(xb[i mod 3]);
        Bound[i].y:=round(yb[i mod 3]);
      end;
      if FormatDialog.CBFill.Checked then begin
        Brush.Color:=FormatDialog.FillBox.Brush.Color;
        Windows.Polygon(Handle,Bound,3);
      end
      else
        Windows.Polyline(Handle,Bound,4);
    end;
    // Grid lines
    if FormatDialog.CBGrid.Checked then begin
      Pen.Width:=FormatDialog.SEGridW.Value;
      Pen.Color:=FormatDialog.GridBox.Brush.Color;
      DrawGridDir(Canvas,xb[0],yb[0],xb[1],yb[1],xb[2],yb[2],split);
      DrawGridDir(Canvas,xb[1],yb[1],xb[2],yb[2],xb[0],yb[0],split);
      DrawGridDir(Canvas,xb[2],yb[2],xb[0],yb[0],xb[1],yb[1],split);
    end;
    // Isolines
    Pen.Width:=FormatDialog.SELineW.Value;
    Pen.Color:=FormatDialog.LineBox.Brush.Color;
    for i:=0 to pp^.splits-1 do begin
      if pp^.wires = nil then continue; // FIXME ... but who cares?
      for j:=0 to pp^.wires[i]^.numchains-1 do begin
        clist:=pp^.wires[i]^.clist^[j];
        MoveTo(
          round(pp^.wires[i]^.vlist^[clist^[0]][0] * zoomx + shiftx),
          round(pp^.wires[i]^.vlist^[clist^[0]][1] * zoomy + shifty));
        for k:=1 to pp^.wires[i]^.numvert^[j]-1 do begin
          LineTo(
            round(pp^.wires[i]^.vlist^[clist^[k]][0] * zoomx + shiftx),
            round(pp^.wires[i]^.vlist^[clist^[k]][1] * zoomy + shifty));
        end; // for k
        // line labels
        if FormatDialog.CBTics.Checked then begin
          Font:=FormatDialog.TicsFontDialog.Font;
          if (pp^.wires[i]^.llist[j].flags and LF_NOLABEL) = 0 then
          begin
            StrCopy(lt,pp^.wires[i]^.llist[j].text);
            // hack - reformat line labels
            if comma <> '.' then
            begin
              plt := lt;
              while plt^ <> #0 do
              begin
                if plt^ = '.' then plt^:=comma;
                plt:=plt + 1;
              end;
            end;
            DrawLabel(Canvas,
              pp^.wires[i]^.llist[j].center[0] * zoomx + shiftx,
              pp^.wires[i]^.llist[j].center[1] * zoomy + shifty,
              PChar(Format(FormatDialog.STicksForm.Text,[StrToFloat(lt)])));
          end;
        end;
      end; // for j
    end; // for i
    // Scales
    if FormatDialog.CBAxes.Checked then begin
      Pen.Width:=FormatDialog.SEAxesW.Value;
      Pen.Color:=FormatDialog.AxesBox.Brush.Color;
      Font:=FormatDialog.LabelFontDialog.Font;
      Brush.Color:=FormatDialog.BackBox.Brush.Color;
      // Right line: from A to B; C is constant
      v1a:=abc^[0][0]; // a -   
      v1b:=abc^[0][1]; // b -   
      v2a:=abc^[1][0]; // a -   
      v2b:=abc^[1][1]; // b -   
      dva:=(v2a-v1a)/(split+1); //   a
      dvb:=(v2b-v1b)/(split+1); //   b
      p1[0]:=xb[0];
      p1[1]:=yb[0];
      p2[0]:=xb[1];
      p2[1]:=yb[1];
      vec2Sub(p,p2,p1);
      ofs:=aoscale * vec2Abs(p)/disdiv;
      len:=tickslen * vec2Abs(p)/disdiv/lendiv;
      vec2PerpendicularLine(d1,d2,da,p1,p2,len,-ofs,split+1);
      Pen.Width:=2;
      MoveTo(round(p1[0]),round(p1[1]));
      LineTo(round(p2[0]),round(p2[1]));
      Ellipse(round(p1[0]-2),round(p1[1]-2),round(p1[0]+2),round(p1[1]+2));
      Ellipse(round(p2[0]-2),round(p2[1]-2),round(p2[0]+2),round(p2[1]+2));
      Pen.Width:=1;
      for i:=1 to split do
      begin
        vec2Add(d1,d1,da);
        vec2Add(d2,d2,da);
        MoveTo(round(d1[0]),round(d1[1]));
        LineTo(round(d2[0]),round(d2[1]));
        if FormatDialog.CBLabels.Checked then begin
          v1a:=v1a+dva;
          v1b:=v1b+dvb;
          s:=Format(FormatDialog.SLabForm.Text,[v1a]);
          TextOut(round(d1[0]),round(d1[1]-TextHeight(s)),s);
          s:=Format(FormatDialog.SLabForm.Text,[v1b]);
          TextOut(round(d2[0]-TextWidth(s)),round(d2[1]),s);
        end;
      end;
      //   ( B  C)
      //    (A)
      v1b:=abc^[1][1]; // b -   
      v1c:=abc^[1][2]; // c -   
      v2b:=abc^[2][1]; // b -   
      v2c:=abc^[2][2]; // c -   
      dvb:=(v2b-v1b)/(split+1); //   b
      dvc:=(v2c-v1c)/(split+1); //   c
      p1[0]:=xb[1];
      p1[1]:=yb[1];
      p2[0]:=xb[2];
      p2[1]:=yb[2];
      vec2Sub(p,p2,p1);
      ofs:=aoscale*vec2Abs(p)/disdiv;
      len:=tickslen * vec2Abs(p)/disdiv/lendiv;
      vec2PerpendicularLine(d1,d2,da,p1,p2,len,-ofs,split+1);
      Pen.Width:=2;
      MoveTo(round(p1[0]),round(p1[1]));
      LineTo(round(p2[0]),round(p2[1]));
      Ellipse(round(p1[0]-2),round(p1[1]-2),round(p1[0]+2),round(p1[1]+2));
      Ellipse(round(p2[0]-2),round(p2[1]-2),round(p2[0]+2),round(p2[1]+2));
      Pen.Width:=1;
      for i:=1 to split do
      begin
        vec2Add(d1,d1,da);
        vec2Add(d2,d2,da);
        MoveTo(round(d1[0]),round(d1[1]));
        LineTo(round(d2[0]),round(d2[1]));
        if FormatDialog.CBLabels.Checked then begin
          v1b:=v1b+dvb;
          v1c:=v1c+dvc;
          s:=Format(FormatDialog.SLabForm.Text,[v1b]);
          TextOut(round(d1[0]-TextWidth(s)/2),round(d1[1]),s);
          s:=Format(FormatDialog.SLabForm.Text,[v1c]);
          TextOut(round(d2[0]-TextWidth(s)/2),round(d2[1]-TextHeight(s)),s);
        end;
      end;
      //  :    A
      //    (B)
      v1a:=abc^[2][0]; // a -   
      v1c:=abc^[2][2]; // c -   
      v2a:=abc^[0][0]; // a -   
      v2c:=abc^[0][2]; // c -   
      dva:=(v2a-v1a)/(split+1); //   a
      dvc:=(v2c-v1c)/(split+1); //   c
      p1[0]:=xb[2];
      p1[1]:=yb[2];
      p2[0]:=xb[0];
      p2[1]:=yb[0];
      vec2Sub(p,p2,p1);
      ofs:=aoscale*vec2Abs(p)/disdiv;
      len:=tickslen * vec2Abs(p)/disdiv/lendiv;
      vec2PerpendicularLine(d1,d2,da,p1,p2,len,-ofs,split+1);
      Pen.Width:=2;
      MoveTo(round(p1[0]),round(p1[1]));
      LineTo(round(p2[0]),round(p2[1]));
      Ellipse(round(p1[0]-2),round(p1[1]-2),round(p1[0]+2),round(p1[1]+2));
      Ellipse(round(p2[0]-2),round(p2[1]-2),round(p2[0]+2),round(p2[1]+2));
      Pen.Width:=1;
      for i:=1 to split do
      begin
        vec2Add(d1,d1,da);
        vec2Add(d2,d2,da);
        MoveTo(round(d1[0]),round(d1[1]));
        LineTo(round(d2[0]),round(d2[1]));
        if FormatDialog.CBLabels.Checked then begin
          v1c:=v1c+dvc;
          v1a:=v1a+dva;
          s:=Format(FormatDialog.SLabForm.Text,[v1c]);
          TextOut(round(d1[0]-TextWidth(s)),round(d1[1]-TextHeight(s)),s);
          s:=Format(FormatDialog.SLabForm.Text,[v1a]);
          TextOut(round(d2[0]),round(d2[1]),s);
        end;
      end;
      // Labels
      Font:=FormatDialog.PointsFontDialog.Font;
      if FormatDialog.LabelA.Text <> '' then
        TextOut(round(xb[0]-TextWidth(FormatDialog.LabelA.Text)/2),round(yb[0] - TextHeight(FormatDialog.LabelA.Text) - LabelOffset),FormatDialog.LabelA.Text);
      if FormatDialog.LabelB.Text <> '' then
        TextOut(round(xb[1] + LabelOffset),round(yb[1]),FormatDialog.LabelB.Text);
      if FormatDialog.LabelC.Text <> '' then
        TextOut(round(xb[2]-TextWidth(FormatDialog.LabelC.Text) - LabelOffset),round(yb[2]),FormatDialog.LabelC.Text);
    end;
  end; // with
end;

end.

